'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var readPkg = _interopDefault(require('read-pkg'));
var camelcase = _interopDefault(require('camelcase'));
var path = require('path');
var builtinModules = _interopDefault(require('builtin-modules'));
var lodash = require('lodash');
var addShebang = _interopDefault(require('rollup-plugin-add-shebang'));
var babel = _interopDefault(require('rollup-plugin-babel'));
var buble = _interopDefault(require('rollup-plugin-buble'));
var commonjs = _interopDefault(require('rollup-plugin-commonjs'));
var exportEquals = _interopDefault(require('rollup-plugin-export-equals'));
var json = _interopDefault(require('rollup-plugin-json'));
var nodeResolve = _interopDefault(require('rollup-plugin-node-resolve'));
var stripShebang = _interopDefault(require('rollup-plugin-strip-shebang'));
var rollupPluginTerser = require('rollup-plugin-terser');
var typescript2 = _interopDefault(require('rollup-plugin-typescript2'));

function error(msg) {
  return new TypeError(msg);
}
function invalidPkgField(field, type) {
  return error(("Invalid package.json \"" + field + "\" field. It has to be of type " + type));
}
function invalidOption(option, type) {
  return error(("Invalid \"" + option + "\" option. It has to be " + type));
}
function noTsInput(desc) {
  return error((desc + " input has to point to a typescript (.ts) file."));
}

var keys = Object.keys;

function keysOrNull(deps) {
  return deps ? keys(deps) : null;
}

function isBrowserFormat(value) {
  return !!value && (value === "iife" || value === "amd" || value === "umd");
}

function setProp(name, value, target) {
  target[name] = value;
  return target;
}

function isNull(value) {
  return value == null;
}
function isObject(value) {
  return !!value && typeof value === "object";
}
function isString(value) {
  return typeof value === "string";
}
function isBool(value) {
  return value === true || value === false;
}
var isArray = Array.isArray;
function isDictionary(value) {
  return isObject(value) && !isArray(value);
}
function isStringOrNull(value) {
  return isNull(value) || isString(value);
}
function isDictionaryOrNull(value) {
  return isNull(value) || isDictionary(value);
}

function isValidGlobals(value) {
  return isNull(value) || isObject(value) && (isArray(value) ? value.every(function (item) { return isString(item); }) : keys(value).every(function (key) { return isString(key) && isString(value[key]); }));
}
function normalizeGlobals(globals) {
  return !globals ? null : isArray(globals) ? globals.reduce(function (result, value) { return setProp(value, value, result); }, {}) : globals;
}

function isOutputField(value) {
  return !!value && (value === "main" || value === "module" || value === "browser");
}
function isValidMinOption(value) {
  return isNull(value) || isBool(value) || isOutputField(value) || isArray(value) && value.every(function (item) { return isOutputField(item); });
}
function baseMinOption(value) {
  return ["main", "module", "browser"].reduce(function (options, field) { return setProp(field, !!value, options); }, {});
}
function normalizeMin(min) {
  return !min ? baseMinOption() : min === true ? baseMinOption(true) : isArray(min) ? min.reduce(function (result, field) { return setProp(field, true, result); }, baseMinOption()) : setProp(min, true, baseMinOption());
}

function normalizeBuildName(cwd, nameOption, pkgName) {
  return nameOption || pkgName && camelcase(path.basename(pkgName)) || camelcase(path.basename(cwd)) || null;
}

function resolve(cwd, filepath) {
  return filepath ? path.resolve(cwd, filepath) : null;
}

function getInvalidOptions(options) {
  return keys(options).filter(function (name) { return name !== "input" && name !== "bin" && name !== "extend" && name !== "esModule" && name !== "interop" && name !== "equals" && name !== "sourcemap" && name !== "format" && name !== "name" && name !== "id" && name !== "globals" && name !== "min" && name !== "cache"; });
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var analizePkg = _async(function (cwd, pkg) {
  return _await(pkg || readPkg({
    cwd: cwd,
    normalize: false
  }), function (resolvedPkg) {
    if (!isDictionary(resolvedPkg)) {
      throw error("Invalid package.json content");
    }

    var pkgName = resolvedPkg.name;
    var cjsModuleFile = resolvedPkg.main;
    var esModuleStFile = resolvedPkg.module;
    var esModuleFbFile = resolvedPkg["jsnext:main"];
    var browserFile = resolvedPkg.browser;
    var binFile = resolvedPkg.bin;
    var pkgTypes = resolvedPkg.types;
    var typings = resolvedPkg.typings;
    var runtimeDependencies = resolvedPkg.dependencies;
    var peerDependencies = resolvedPkg.peerDependencies;
    var optionalDependencies = resolvedPkg.optionalDependencies;
    var bundlibOptions = resolvedPkg.bundlib;

    if (!isStringOrNull(cjsModuleFile)) {
      throw invalidPkgField("main", "string");
    }

    if (!isStringOrNull(esModuleStFile)) {
      throw invalidPkgField("module", "string");
    }

    if (!isStringOrNull(esModuleFbFile) && !esModuleStFile) {
      throw invalidPkgField("jsnext:main", "string");
    }

    if (!isStringOrNull(browserFile)) {
      throw invalidPkgField("browser", "string");
    }

    if (!isStringOrNull(binFile)) {
      throw invalidPkgField("bin", "string");
    }

    if (!isDictionaryOrNull(bundlibOptions)) {
      throw invalidPkgField("bundlib", "Object");
    }

    if (!isDictionaryOrNull(runtimeDependencies)) {
      throw invalidPkgField("dependencies", "Object");
    }

    if (!isDictionaryOrNull(peerDependencies)) {
      throw invalidPkgField("peerDependencies", "Object");
    }

    if (!isDictionaryOrNull(optionalDependencies)) {
      throw invalidPkgField("optionalDependencies", "Object");
    }

    var invalidOptions = bundlibOptions && getInvalidOptions(bundlibOptions);

    if (invalidOptions && invalidOptions.length) {
      var optionNames = invalidOptions.map(function (name) { return ("\"" + name + "\""); }).join(", ");
      throw error(("Unknown options found: (" + optionNames + ")"));
    }

    var ref = bundlibOptions || {};
    var pkgInput = ref.input;
    var pkgBinInput = ref.bin;
    var sourcemapOption = ref.sourcemap;
    var esModuleFlag = ref.esModule;
    var interopFlag = ref.interop;
    var extendFlag = ref.extend;
    var equalsFlag = ref.equals;
    var browserFormat = ref.format;
    var browserName = ref.name;
    var amdId = ref.id;
    var browserGlobals = ref.globals;
    var min = ref.min;
    var cacheOption = ref.cache;

    if (!isStringOrNull(pkgInput)) {
      throw invalidOption("input", "string");
    }

    if (!isStringOrNull(pkgBinInput)) {
      throw invalidOption("bin", "string");
    }

    if (!isNull(sourcemapOption) && !isBool(sourcemapOption) && sourcemapOption !== "inline") {
      throw invalidOption("sourcemap", 'boolean | "inline"');
    }

    if (!isNull(browserFormat) && !isBrowserFormat(browserFormat)) {
      throw invalidOption("format", '"amd" | "iife" | "amd"');
    }

    if (!isStringOrNull(browserName)) {
      throw invalidOption("name", "string");
    }

    if (!isStringOrNull(amdId)) {
      throw invalidOption("id", "string");
    }

    if (!isValidGlobals(browserGlobals)) {
      throw invalidOption("globals", "Object<string, string> | string[]");
    }

    if (!isValidMinOption(min)) {
      throw invalidOption("min", 'boolean | "main" | "module" | "browser" | Array<"main" | "module" | "browser">');
    }

    if (!isStringOrNull(cacheOption)) {
      throw invalidOption("cache", "string");
    }

    var input = {
      api: resolve(cwd, pkgInput || "src/index.ts"),
      bin: resolve(cwd, pkgBinInput || "src-bin/index.ts")
    };
    var esModuleFile = esModuleStFile || esModuleFbFile;
    var typesPath = pkgTypes || typings;
    var output = {
      main: resolve(cwd, cjsModuleFile),
      module: resolve(cwd, esModuleFile),
      browser: resolve(cwd, browserFile),
      bin: resolve(cwd, binFile),
      types: resolve(cwd, typesPath)
    };
    var dependencies = {
      runtime: keysOrNull(runtimeDependencies),
      peer: keysOrNull(peerDependencies),
      optional: keysOrNull(optionalDependencies)
    };
    var minify = normalizeMin(min);
    var buildName = normalizeBuildName(cwd, browserName, pkgName);
    var globals = normalizeGlobals(browserGlobals);
    var browser = {
      format: browserFormat || "umd",
      name: buildName,
      id: amdId || null,
      globals: globals
    };
    var sourcemap = sourcemapOption === "inline" ? "inline" : sourcemapOption !== false;
    var cache = resolve(cwd, cacheOption || "node_modules/.cache/bundlib");
    var options = {
      esModule: !!esModuleFlag,
      interop: !!interopFlag,
      extend: !!extendFlag,
      equals: !!equalsFlag
    };
    return {
      cwd: cwd,
      pkg: resolvedPkg,
      input: input,
      output: output,
      sourcemap: sourcemap,
      dependencies: dependencies,
      minify: minify,
      browser: browser,
      options: options,
      cache: cache
    };
  }, pkg);
});

function _async(f) {
  return function () {
    var arguments$1 = arguments;

    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments$1[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function arrayToExternal(modules) {
  var cache = {};
  return function (source, _, isResolved) {
    if (isResolved || source[0] === ".") {
      return;
    }

    return cache[source] || modules.some(function (moduleName) {
      if (source === moduleName) {
        return cache[source] = true;
      }

      var len = moduleName.length;

      if (source.substr(0, len) === moduleName && source[len].match(/^[/\\]$/)) {
        return cache[source] = true;
      }

      return false;
    });
  };
}

function createOutput(format, file, sourcemap, esModule, interop, extra) {
  var options = {
    file: file,
    format: format,
    sourcemap: sourcemap,
    esModule: esModule,
    interop: interop
  };
  return extra ? Object.assign(options, extra) : options;
}

function createConfig(input, output, external, plugins, chokidar) {
  return {
    input: input,
    output: output,
    external: !external ? function () { return false; } : arrayToExternal(external),
    plugins: plugins.filter(Boolean),
    watch: {
      chokidar: chokidar,
      exclude: ["node_modules/**"]
    }
  };
}
function createModuleConfig(input, format, file, sourcemap, esModule, interop, external, plugins, chokidar) {
  return createConfig(input, createOutput(format, file, sourcemap, esModule, interop), external, plugins, chokidar);
}
function createBrowserConfig(input, format, file, sourcemap, esModule, interop, plugins, chokidar, name, extend, globals, id) {
  var extra = {
    name: name,
    extend: extend,
    globals: globals || {}
  };

  if (id && (format === "umd" || format === "amd")) {
    extra.amd = {
      id: id
    };
  }

  return createConfig(input, createOutput(format, file, sourcemap, esModule, interop, extra), keysOrNull(globals), plugins, chokidar);
}

function renamePre(filename, pre) {
  var ref = path.parse(filename);
  var dir = ref.dir;
  var name = ref.name;
  var ext = ref.ext;
  return path.join(dir, (name + "." + pre + ext));
}
function renameMin(filename) {
  return renamePre(filename, "min");
}

function pkgToConfigs(ref, dev) {
  var cwd = ref.cwd;
  var inputObject = ref.input;
  var output = ref.output;
  var sourcemap = ref.sourcemap;
  var minify = ref.minify;
  var dependencies = ref.dependencies;
  var browserOptions = ref.browser;
  var options = ref.options;
  var cacheFolder = ref.cache;

  var apiInput = inputObject.api;
  var binInput = inputObject.bin;
  var cjsOutputFile = output.main;
  var esOutputFile = output.module;
  var browserOutputFile = output.browser;
  var binaryOutputFile = output.bin;
  var typesOutputFile = output.types;

  if ((cjsOutputFile || esOutputFile || browserOutputFile) && path.extname(apiInput) !== ".ts") {
    throw noTsInput("Module");
  }

  if (binaryOutputFile && path.extname(binInput) !== ".ts") {
    throw noTsInput("Binary");
  }

  var runtimeDeps = dependencies.runtime;
  var peerDeps = dependencies.peer;
  var optionalDeps = dependencies.optional;
  var esModule = options.esModule;
  var interop = options.interop;
  var extend = options.extend;
  var equals = options.equals;
  var browserFormat = browserOptions.format;
  var pkgName = browserOptions.name;
  var id = browserOptions.id;
  var globals = browserOptions.globals;
  var nameNeeded = browserOutputFile && (browserFormat === "iife" || browserFormat === "umd");

  if (nameNeeded && !pkgName) {
    throw error("name option is required for IIFE and UMD builds");
  }

  var prod = !dev;
  var apiFolder = path.dirname(apiInput);
  var apiFolderContent = path.resolve(apiFolder, "**/*.ts");
  var cwdFolderContent = path.resolve(cwd, "**/*.ts");
  var typesFilename = renamePre(path.basename(apiInput), "d");
  var sourcemapBool = !!sourcemap;
  var typesOutputDir = typesOutputFile;

  if (typesOutputDir && path.extname(typesOutputDir) === ".ts") {
    typesOutputDir = path.dirname(typesOutputDir);
  }

  var external = lodash.union(runtimeDeps, peerDeps, optionalDeps, builtinModules);
  var extensions = [".ts", ".js"];
  var exclude = /node_modules/;
  var configs = [];

  function createPlugins(browser, mini, bin) {
    var declarationDir = !configs.length && !bin && typesOutputDir;
    var tsInclude = bin ? [cwdFolderContent] : [apiFolderContent];
    var cacheRoot = path.join(cacheFolder, "rpt2");
    var shebang;
    return [!!bin && stripShebang({
      capture: function (shebangFromFile) { return shebang = shebangFromFile; },
      sourcemap: sourcemapBool
    }), !!bin && !!cjsOutputFile && {
      name: "api",

      resolveId: function resolveId(moduleId, from) {
        var resolved = !from ? moduleId : path.join(path.dirname(from), moduleId);

        if (resolved === apiInput || path.join(resolved, ".ts") === apiInput || path.join(resolved, "/index.ts") === apiInput) {
          return {
            id: path.relative(path.dirname(bin), cwd),
            external: true,
            moduleSideEffects: false
          };
        }

        return null;
      }

    }, browser && nodeResolve({
      preferBuiltins: !browser,
      extensions: extensions
    }), browser && commonjs({
      sourceMap: sourcemapBool
    }), typescript2({
      include: tsInclude,
      cacheRoot: cacheRoot,
      useTsconfigDeclarationDir: true,
      tsconfigDefaults: {
        include: tsInclude,
        exclude: []
      },
      tsconfigOverride: {
        compilerOptions: {
          target: "esnext",
          module: "esnext",
          moduleResolution: "node",
          sourceMap: sourcemapBool,
          declaration: !!declarationDir,
          declarationDir: declarationDir || "",
          emitDeclarationOnly: false
        }
      }
    }), json(), !!declarationDir && equals && exportEquals({
      file: path.resolve(cwd, path.join(declarationDir, typesFilename))
    }), babel({
      extensions: extensions,
      exclude: exclude,
      babelrc: false,
      plugins: [require.resolve("babel-plugin-transform-async-to-promises")]
    }), buble({
      exclude: exclude,
      target: {
        node: 0.12
      },
      objectAssign: true
    }), !!bin && addShebang({
      include: bin,
      shebang: function () { return shebang; }
    }), mini && rollupPluginTerser.terser({
      sourcemap: sourcemapBool,
      toplevel: true,
      module: true
    })];
  }

  if (esOutputFile) {
    configs.push(createModuleConfig(apiInput, "es", esOutputFile, sourcemap, true, false, external, createPlugins(false, prod && !minify.module), true));

    if (minify.module) {
      configs.push(createModuleConfig(apiInput, "es", renameMin(esOutputFile), sourcemap, true, false, external, createPlugins(false, true), true));
    }
  }

  if (cjsOutputFile) {
    configs.push(createModuleConfig(apiInput, "cjs", cjsOutputFile, sourcemap, esModule, interop, external, createPlugins(false, prod && !minify.main), true));

    if (minify.main) {
      configs.push(createModuleConfig(apiInput, "cjs", renameMin(cjsOutputFile), sourcemap, esModule, interop, external, createPlugins(false, true), true));
    }
  }

  if (browserOutputFile) {
    configs.push(createBrowserConfig(apiInput, browserFormat, browserOutputFile, sourcemap, esModule, interop, createPlugins(true, prod && !minify.browser), true, pkgName, extend, globals, id));

    if (minify.browser) {
      configs.push(createBrowserConfig(apiInput, browserFormat, renameMin(browserOutputFile), sourcemap, esModule, interop, createPlugins(true, true), true, pkgName, extend, globals, id));
    }
  }

  if (binaryOutputFile) {
    configs.push(createModuleConfig(binInput, "cjs", binaryOutputFile, sourcemap, esModule, interop, external, createPlugins(false, prod, binaryOutputFile), true));
  }

  return configs;
}

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var configsFromPkg = _async$1(function (cwd, dev, pkgJson) {
  return _await$1(analizePkg(cwd, pkgJson), function (_analizePkg) {
    return pkgToConfigs(_analizePkg, dev);
  });
});

function _async$1(f) {
  return function () {
    var arguments$1 = arguments;

    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments$1[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

exports.analizePkg = analizePkg;
exports.configsFromPkg = configsFromPkg;
//# sourceMappingURL=bundlib.cjs.js.map
